#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
package ${package}.infra;

import javax.enterprise.context.spi.CreationalContext;
import javax.enterprise.inject.spi.Bean;
import javax.enterprise.inject.spi.BeanManager;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * このクラスは、CDIのAPIを簡易に使用するためのメソッドを提供するユーティリティです。
 * @author SIToolkit
 */
public class CDIUtils {
	
	private static final Logger LOG = LoggerFactory.getLogger(CDIUtils.class);
	private static final String BEAN_MANAGER_JNDI = "java:comp/BeanManager";

	/**
	 * CDIコンテキストからビーンを取得します。
	 * @param <T> ビーンの型
	 * @param type ビーンの型のクラス
	 * @return CDIコンテキスト内のビーン
	 */
	public static <T> T getBean(Class<T> type) {
		BeanManager bm = getBeanManager();
		if (bm == null) {
			return null;
		}
		Bean<T> bean = (Bean<T>) bm.getBeans(type).iterator().next();
		CreationalContext<T> ctx = bm.createCreationalContext(bean);
		return (T) bm.getReference(bean, type, ctx);
	}

	/**
	 * コンテナからBeanManagerインスタンスを取得します。
	 * BeanManagerインスタンスが見つからない場合はnullを返します。
	 * @return BeanManagerインスタンス
	 */
	public static BeanManager getBeanManager() {
		InitialContext ctx;
		try {
			ctx = new InitialContext();
			return (BeanManager) ctx.lookup(BEAN_MANAGER_JNDI);
		} catch (NamingException e) {
			LOG.warn("BeanManagerのルックアップが失敗しました。", e);
			return null;
		}
	}

}
